$TITLE                       MODEL EXTERplusGLS
$STITLE     ENDOGENOUS MALE AND FEMALE LABOUR SUPPLY IN EXTERplus MODEL

*Model of a small open economy with government producing 4 goods using 4 factors
*owned by 4 households.
*2 labour factors hold by male and female.
*Endogenous male and female labour supply.

*==================================CALIBRATION===============================*


*--------------------------------Sets definition-----------------------------*

SET I Sectors           /  AGR   agriculture
                           IND   industry
                           SER   services
                           NTSER non tradable services /


TR(I) Tradable          /  AGR   agriculture
                           IND   industry
                           SER   services     /

NAG(TR) Non-agricultural / IND   industry
                           SER   services     /

AGRS(TR) Agricultural   /  AGR   agriculture /

NTR(I) Non tradable     /  NTSER non tradable services /


GOOD(TR) Goods          /  AGR agriculture
                           IND industry   /


H Households            /  RP   rural small-no land
                           UP   urban uneducated
                           RR   rural large land
                           UR   urban educated /

ALIAS (I,J)
ALIAS (TR, TRJ)


*-----------------------------Parameters definition--------------------------*

PARAMETERS

  A_KL(TR)          Scale parameter (CES between labor and capital)
  alpha_kl(TR)      Share parameter (CES between labor and capital)
  sigma_kl(TR)      Substitution elasticity (CES capital labor)
  rho_kl(TR)        Substitution parameter (CES capital labor)
  A_CL              Scale parameter (CES between composite factor and land)
  alpha_cl          Share parameter (CES between composite factor and land)
  sigma_cl          Substitution elasticity (CES composite factor land)
  rho_cl            Substitution parameter (CES composite factor land)
  io(I)             Coefficient (Leontief total intermediate consumption)
  v(i)              Coefficient (Leontief value added)
  aij(TR,J)         Input output coefficient
  gamma(TR,H)       Marginal share of good TR in LES consumption function
  YELAS(TR,H)       Income elasticity of good TR
*===============================================================================
  LELAS_fem(H)      Household H elasticity of female labor supply with respect to income
  ZETA_fem(H)       Household H share of female leisure consumption
  MAXHOUR_fem(H)    Household H female work-leisure maximum hours

  LELAS_mal(H)      Household H elasticity of male labor supply with respect to income
  ZETA_mal(H)       Household H share of male leisure consumption
  MAXHOUR_mal(H)    Household H male work-leisure maximum hours
*===============================================================================
  V_MIN(H)          Minimum consumption value (temp variable)
  C_MIN(TR,H)       Minimum consumption of good TR (LES consumption function)
  FRISCH(H)         Frisch parameter (LES consumption function)
  psi(H)            Propensity to save for household H
  mu(TR)            Share of the value of good TR in total investment
  lambda_l(H)       Share of land income received by household H
  lambda_lf         Share of land income received by firms
  lambda_lrw        Share of land income received by foreigners
  lambda_r(H)       Share of capital income received by household H
  lambda_rf         Share of capital income received by firms
  lambda_row        Share of capital income received by foreigners
*-------------------------------------------------------------------------------
*lambda_w(H)        Share of labor income received by household H
*lambda_wf(H)       Share of female labor income received by household H
*lambda_wm(H)       Share of male labor income received by household H
*-------------------------------------------------------------------------------
  te(TR)            Tax on exports on good TR
  tm(TR)            Import duties on good TR
  tx(TR)            Tax rate on good TR
  tyh(H)            Direct income tax rate for household H
  tyf               Direct income tax rate for firms
  B_E(TR)           Scale parameter (CET function)
  beta_e(TR)        Share parameter (CET function)
  kappa_e(TR)       Transformation parameter (CET function)
  tau_e(TR)         Transformation elasticity (CET function)
  A_M(TR)           Scale parameter (CES function)
  rho_m(TR)         Substitution parameter (CES function)
  alpha_m(TR)       Share parameter (CES function)
  sigma_m(TR)       Substitution elasticity (CES function)
  delta(I)          Share of sector I in total value added
  deltak(TR)        Share of product TR in total capital
*-------------------------------------------------------------------------------
  A_lab(J)      Scale Parameter in aggregated labour factor
  sigma_lab(J)        Substitution elasticity of gender labour
  rho_lab(J)      Substitution parameter in  gender labour
  alpha_lab(J)     Share parameter in agregated labour
*-------------------------------------------------------------------------------

*Prices

  wo                Wage rate
*-------------------------------------------------------------------------------
  WfemO             Femaler wage rate
  WmalO             Male wage rate
*-------------------------------------------------------------------------------
  ro(TR)            Rate of return to capital
  rlo               Rate of return to agricultural land
  rco               Rate or return to composite agricultural capital-labor
  PO(I)             Producer price of good I
  PVO(I)            Value added price for sector I
  PDO(TR)           Domestic price of good TR including tax
  PLO(TR)           Domestic price of good TR excluding tax
  PCO(TR)           Price of composite good TR
  PMO(TR)           Domestic price of imported good TR
  PEO(TR)           Domestic price of exported good TR
  PWMO(TR)          World price of import TR (foreign currency)
  PWEO(TR)          World price of export TR (foreign currency)
  PINDEXO           Producer price index
  PINVO             Producer price index
  eo                Exchange rate

*Production
  XSO(I)            Production of sector I
  VAO(I)            Value added in sector I (volume)
  DIO(TR,J)         Intermediate consumption of good TR in sector J
  CIO(I)            Total intermediate consumption of sector I

*Factors
  KSO               Total capital supply
  KDO(TR)           Sector TR demand for capital
  LANDO             Agricultural land
  LDO(I)            Sector I demand for labour
  CFO               Composite agricultural capital-labor factor
*LSO               Total labour supply
*-------------------------------------------------------------------------------
*  LSfemO            Female Total labour supply
*  LSmalO            Male Total labour supply
*===============================================================================
  LSfemO(H)         Household H female labour supply
  LSmalO(H)         Household H male labour supply
  LSTfemO           Total female labour supply
  LSTmalO           Total male labour supply
*===============================================================================
  LDfemO(I)         Sector I demand for female labour
  LDmalO(I)         Sector I demand for male labour
*-------------------------------------------------------------------------------

*Demand
  CO(TR,H)          Consumption of good TR by household H (volume)
  CTHO(H)           Household H total consumption
  INVO(TR)          Investment in good TR (volume)
  ITO               Total investment (value)
  ITVOLO            Total investment (volume)
  DITO(TR)          Intermediate demand for good TR
  GO                Total public consumption (value)
  DO(TR)            Demand  for domestic good TR
  QO(TR)            Demand for composite good TR

*International trade
  MO(TR)            Imports of good TR
  EXO(TR)           Exports of good TR
  CABO              Current account balance

*Income and savings
  YHO(H)            Household H income
  YDHO(H)           Household H disposable income
  YFO               Firms income
  YGO               Government income
  SHO(H)            Household H savings
  SFO               Firms savings
  SGO               Government savings
  DIVO(H)           Dividends paid to capitalist households
  DIV_ROWO          Dividends paid to foreigners
  TGO(H)            Public transfers to households
  TIO(TR)           Receipts from indirect tax
  TIMO(TR)          Receipts from indirect tax on imports
  TIEO(TR)          Receipts from indirect tax on exports
  DTHO(H)           Receipts from direct income taxation for household H
  DTFO              Receipts from direct income taxation on firms
;

*-------------------------------Benchmarck data------------------------------*
$include data_gls.gms

SCALAR

wo             /    1.0 /
*---------------------------
wfemo          /    1.0 /
wmalo          /    1.0 /
*---------------------------
rco            /    1.0 /
YFO            / 4975.0 /
SFO            / 1410.0 /
YGO            /10560.0 /
SGO            / 1710.0 /
GO             / 8700.0 /
ITO            / 9565.0 /
DTFO           / 1295.0 /
DIV_ROWO       /  370.0 /
lambda_row     /   40.0 /
lambda_lrw     /  885.0 /
sigma_cl       /    1.5 /
eo             /    1.0 /
;

*-----------------------------Variable assignment----------------------------*

XSO(I)         = DP("XSO",I);
VAO(I)         = DP("VAO",I);
KDO(TR)        = DP("KDO",TR);
*LDO(I)         = DP("LDO",I);
LANDO          = DP("LANDO","AGR");
INVO(TR)       = DP("INVO",TR);
EXO(TR)        = DP("EXO",TR);
DO(TR)         = DP("DO",TR);
MO(TR)         = DP("MO",TR);
QO(TR)         = DP("QO",TR);
ro(TR)         = DP("ro",TR);
rlo            = DP("rlo","AGR");
PO(I)          = DP("PO",I);
PLO(TR)        = DP("PLO",TR);
PWMO(TR)       = DP("PWMO",TR);
PEO(TR)        = DP("PEO",TR);
TIO(TR)        = DP("TIO",TR);
TIMO(TR)       = DP("TIMO",TR);
TIEO(TR)       = DP("TIEO",TR);
sigma_m(TR)    = DP("sigma_m",TR);
sigma_kl(TR)   = DP("sigma_kl",TR);
tau_e(TR)      = DP("tau_e",TR);
YHO(H)         = HOUSEHOLDS("YHO",H);
SHO(H)         = HOUSEHOLDS("SHO",H);
DTHO(H)        = HOUSEHOLDS("DTHO",H);
*lambda_w(H)    = HOUSEHOLDS("lambda_w",H);
lambda_r(H)    = HOUSEHOLDS("lambda_r",H);
lambda_l(H)    = HOUSEHOLDS("lambda_l",H);
TGO(H)         = HOUSEHOLDS("TGO",H);
DIVO(H)        = HOUSEHOLDS("DIVO",H);
FRISCH(H)      = HOUSEHOLDS("FRISCH",H);
*-------------------------------------------------------------------------------
sigma_lab(I)      = NEW1("sigma_lab",I);
LDfemO(I)       = NEW1("LDfemO",I);
LDmalO(I)       = NEW1("LDmalO",I);
*lambda_wm(H)    = NEW2("lambda_wm",H);
*lambda_wf(H)    = NEW2("lambda_wf",H);
*===============================================================================
LSmalO(H)       = NEW2("lambda_wm",H);
LSfemO(H)       = NEW2("lambda_wf",H);
LELAS_mal(H)    = NEW2("LELAS_mal",H);
LELAS_fem(H)    = NEW2("LELAS_fem",H);
*===============================================================================

*-----------------------Calculations of other variables----------------------*

*Tax rates

tx(TR)         = TIO(TR)/(DO(TR)+MO(TR)+TIMO(TR));
tyh(H)         = DTHO(H)/YHO(H);
tyf            = DTFO/YFO;
tm(TR)         = TIMO(TR)/MO(TR);
te(TR)         = TIEO(TR)/EXO(TR);

*Prices
PDO(TR)        = ( (1+tx(TR))*(PO(TR)*XSO(TR) - EXO(TR)*PEO(TR)) )/DO(TR) ;
PMO(TR)        = PWMO(TR)*(1+tx(TR))*(1+tm(TR))*eo;
PWEO(TR)       = PEO(TR)*(1+te(TR))/eo;
PCO(TR)        = ( PDO(TR)*DO(TR) + PMO(TR)*MO(TR) )/QO(TR) ;


*Volume calculations
*-------------------------------
LDfemO(I)      = LDfemO(I)/wfemO;
LDmalO(I)      = LDmalO(I)/wmalO;
LDO(I)         = LDfemO(I) + LDmalO(I);
*---------------------------------
*LDO(I)         = LDO(I)/WO ;
KDO(TR)        = KDO(TR)/ro(TR);
LANDO          = LANDO/RLO;
CFO            = LDO("AGR")+KDO("AGR");
XSO(I)         = XSO(I)/PO(I);
DIO(TR,J)      = DIO(TR,J)/PCO(TR);
CO(TR,H)       = CO(TR,H)/PCO(TR);
INVO(TR)       = INVO(TR)/PCO(TR);
DITO(TR)       = SUM(J,DIO(TR,J));
CIO(J)         = SUM(TR,DIO(TR,J));
KSO            = SUM(TR,KDO(TR));
*======================================
LSTfemO         = SUM(H,LSfemO(H));
LSTmalO         = SUM(H,LSmalO(H));
*======================================
YDHO(H)        = YHO(H)-DTHO(H);
EXO(TR)        = EXO(TR)/PEO(TR);

*Value added price

PVO(I)         = ( PO(I)*XSO(I) - SUM(TRJ,PCO(TRJ)*DIO(TRJ,I)) )/VAO(I) ;

*-------------------------Calibration of parameters--------------------------*

*Production (CES and Leontief)
io(I)          = CIO(I)/XSO(I);
v(i)           = VAO(I)/XSO(I);

**CES between capital-labor
rho_kl(TR)     = (1-sigma_kl(TR))/sigma_kl(TR);
alpha_kl(TR)   = (wo/ro(TR))*(LDO(TR)/KDO(TR))**(1/sigma_kl(TR));
alpha_kl(TR)   = alpha_kl(TR)/(1+alpha_kl(TR));
A_KL(NAG)      = VAO(NAG)/(alpha_kl(NAG)*LDO(NAG)**(-rho_kl(NAG))
                 +(1-alpha_kl(NAG))*KDO(NAG)**(-rho_kl(NAG)))
                 **(-1/rho_kl(NAG));
A_KL("AGR")    = CFO/(alpha_kl("AGR")*LDO("AGR")**(-rho_kl("AGR"))
                 +(1-alpha_kl("AGR"))*KDO("AGR")**(-rho_kl("AGR")))
                 **(-1/rho_kl("AGR"));

**CES between land and composite factor
rho_cl         = (1-sigma_cl)/sigma_cl;
alpha_cl       = (rco/rlo)*(CFO/LANDO)**(1/sigma_cl);
alpha_cl       = alpha_cl/(1+alpha_cl);
A_CL           = VAO("AGR")/(alpha_cl*CFO**(-rho_cl)
                 +(1-alpha_cl)*LANDO**(-rho_cl))
                 **(-1/rho_cl);


*Share parameters
psi(H)         = SHO(H)/YDHO(H);
mu(TR)         = (PCO(TR)*INVO(TR))/ITO;
aij(TR,J)      = DIO(TR,J)/CIO(J);
*lambda_w(H)    = lambda_w(H)/SUM(I,LDO(I)*wO);
*-------------------------------------------------------------------------------
*lambda_wm(H)    = lambda_wm(H)/SUM(I,LDmalO(I)*wmalO);
*lambda_wf(H)    = lambda_wf(H)/SUM(I,LDfemO(I)*wfemO);
*-------------------------------------------------------------------------------
lambda_r(H)    = lambda_r(H)/SUM(TR,ro(TR)*KDO(TR));
lambda_row     = lambda_row/SUM(TR,ro(TR)*KDO(TR));
lambda_l(H)    = lambda_l(H)/(rlo*LANDO);
lambda_lrw     = lambda_lrw/(rlo*LANDO);
lambda_rf      = 1-SUM(H,lambda_r(H))-lambda_row;
lambda_lf      = 1-SUM(H,lambda_l(H))-lambda_lrw;
delta(I)       = PVO(I)*VAO(I)/SUM(J,PVO(J)*VAO(J));
deltak(TR)     = ro(TR)*KDO(TR)/SUM(TRJ,ro(TRJ)*KDO(TRJ));

* LES parameters
**Adjustment of the elasticities in order to respect Engel agregation

CTHO(H)  = YDHO(H)-SHO(H);
YELAS(TR,H)  = YELAS(TR,H)/(SUM(TRJ,YELAS(TRJ,H)*PCO(TRJ)*CO(TRJ,H))/CTHO(H));

**Other parameters
*===============================================================================
 ZETA_mal(H) = LSmalO(H)*WmalO*LELAS_mal(H)/(LSmalO(H)*WmalO*LELAS_mal(H)
                 -LSfemO(H)*WfemO*LELAS_fem(H)-CTHO(H)) ;
 ZETA_fem(H) = LSfemO(H)*WfemO*LELAS_fem(H)/(LSmalO(H)*WmalO*LELAS_mal(H)
                 -LSfemO(H)*WfemO*LELAS_fem(H)-CTHO(H)) ;
*===============================================================================

  gamma(TR,H)  = PCO(TR)*CO(TR,H)*(1-ZETA_fem(H)-ZETA_mal(H))*YELAS(TR,H)/CTHO(H);
  V_MIN(H)     = (SUM(TR,PCO(TR)*CO(TR,H)))*(1+1/FRISCH(H));
  C_MIN(TR,H)  = CO(TR,H)-gamma(TR,H)/(1-ZETA_fem(H)-ZETA_mal(H))*(CTHO(H)-V_MIN(H))/PCO(TR);

*===============================================================================
MAXHOUR_mal(H) = LSmalO(H)+(ZETA_mal(H)/WmalO)*(CTHO(H)-V_MIN(H))
                 /(1-ZETA_fem(H)-ZETA_mal(H));
MAXHOUR_fem(H) = LSfemO(H)+(ZETA_fem(H)/WfemO)*(CTHO(H)-V_MIN(H))
                 /(1-ZETA_fem(H)-ZETA_mal(H));
*===============================================================================

*Other prices
PINDEXO        = SUM(I,PVO(I)*delta(I));
PO(TR)         = (PDO(TR)*DO(TR)/(1+tx(TR))+PEO(TR)*EXO(TR) ) /XSO(TR);

*International trade

*Exports (CET)
kappa_e(TR)    = (1+tau_e(TR))/tau_e(TR);
beta_e(TR)     = 1/(1+(PlO(TR))/PEO(TR)*(EXO(TR)/DO(TR))
                  **(kappa_e(TR)-1));
B_E(TR)        = XSO(TR)/(beta_e(TR)*EXO(TR)**kappa_e(TR)
                +(1-beta_e(TR))*DO(TR)**kappa_e(TR))**(1/kappa_e(TR));

*Imports (CES)
rho_m(TR)      = (1-sigma_m(TR))/sigma_m(TR);
alpha_m(TR)    = (PMO(TR)/PDO(TR))*(MO(TR)/DO(TR))**(1/sigma_m(TR));
alpha_m(TR)    = alpha_m(TR)/(1+alpha_m(TR));
A_M(TR)        = QO(TR)/(alpha_m(TR)*MO(TR)**(-rho_m(TR))
                 +(1-alpha_m(TR))*DO(TR)**(-rho_m(TR)))**(-1/rho_m(TR));

*-Labour------------------------------------------------------------------------
rho_lab(I)      = (1-sigma_lab(I))/sigma_lab(I);
alpha_lab(I)    = (LDfemO(I)/LDmalO(I))**(1/sigma_lab(I))*(WfemO/WmalO);
alpha_lab(I)    =alpha_lab(I)/(1+alpha_lab(I));
A_lab(I)     = LDO(i)/(alpha_lab(I)*LDfemO(I)**(-rho_lab(I))
                 +(1-alpha_lab(i))*LDmalO(I)**(-rho_lab(i)))**(-1/rho_lab(I));
*-------------------------------------------------------------------------------

*Current account balance
CABO           = eo*SUM(TR,PWMO(TR)*MO(TR)) +lambda_row*SUM(TR,ro(TR)*KDO(TR))
                 +lambda_lrw*rlo*LANDO + DIV_ROWO-eo*SUM(TR,PWEO(TR)*EXO(TR));


PINVO       = PROD(TR$(INVO(TR) NE 0),(PCO(TR)/mu(TR))**mu(TR));
ITVOLO      = ITO/PINVO;

*---------------------Parameters to display in output file-------------------*

*$include gls_calib_tables.gms

*==============================END OF CALIBRATION============================*
*$ontext;
*====================================MODEL===================================*

*-----------------------------Variables definition---------------------------*

VARIABLES

*Prices
  w             Wage rate
*--------------------------------
  wfem          Female Wage rate
  wmal          Male Wage rate
*--------------------------------
  r(TR)         Rate of return to capital in sector TR
  rl            Rate of return to agricultural land
  rf            Uniform return to capital
  rc            return to composite factor
  P(I)          Producer price of good I
  PD(TR)        Domestic price of good TR including tax
  PV(I)         Value added price for sector I
  PL(TR)        Domestic price of good TR excluding tax
  PC(TR)        Price of composite good TR
  PM(TR)        Domestic price of imported good TR
  PE(TR)        Domestic price of exported good TR
  PWM(TR)       World price of import TR (foreign currency)
  PWE(TR)       World price of export TR (foreign currency)
  PINDEX        Producer price index
  PINV          Price index of investment
  e             Exchange rate

*Production
  XS(I)         Production of sector I
  VA(I)         Value added in sector I (volume)
  DI(TR,J)      Intermediate consumption of good TR in sector J
  CI(I)         Total intermediate consumption of sector I

*Factors
  KD(TR)        Sector TR demand for capital
  KS            Total capital supply
  LD(I)         Sector I demand for labour
  LAND          Agricultural land
  CF            Composite agricultural capital-labor factor
  LDfem(I)      Sector I demand for female labour
  LDmal(I)      Sector I demand for male labour
*  LS            Total labour supply
*-------------------------------------------------------------------------------
*  LSfem         Female labour supply
*  LSmal         Male labour supply
*===============================================================================
  LSfem(H)      Household female labour supply
  LSmal(H)      Household male labour supply
  LSTfem        Total female labour supply
  LSTmal        Total male labour supply
*===============================================================================

*Demand
  C(TR,H)       Household H consumption of good TR (volume)
  CTH(H)        Household H total consumption (value)
  INV(TR)       Investment in good TR (volume)
  IT            Total investment (value)
  ITVOL         Total investment (volume)
  DIT(TR)       Intermediate demand for good TR
  G             Total public consumption (value)
  D(TR)         Demand for domestic good TR
  Q(TR)         Demand for composite good TR

*International trade
  M(TR)         Imports of good TR
  EX(TR)        Exports of good TR
  CAB           Current account balance

*Income and savings
  YH(H)         Household H income
  YDH(H)        Household H disposable income
  YF            Firms income
  YG            Government income
  SH(H)         Household H savings
  SF            Firms savings
  SG            Government savings
  DIV(H)        Dividends paid to capitalist households
  DIV_ROW       Dividends paid to foreigners
  TG(H)         Public transfers to households
  TI(TR)        Receipts from indirect tax
  TIM(TR)       Receipts from import duties
  TIE(TR)       Receipts from tax on exports
  DTH(H)        Receipts from direct taxation on household H income
  DTF           Receipts from direct taxation on firms income
  nu            Adjustment variable for hh savings
  adj           Adjustment variable for indirect taxes

*Others
  EV(H)         Equivalent variation for household H
  LEON          Walras law verification variable
  OMEGA         Objective variable
;


*--------------------------------Equations definition------------------------*

EQUATIONS

*Production
 SUPPLY(I)     Production function for sector I
 VAD1(NAG)     Value added in non-agricultural sectors
 VAD2          Value added in agricultural sectors
 VAD3(NTR)     Value added in non-tradable sectors
 ECF           Composite agricultural labor-capital factor
 CIEQ(I)       Total intermediate consumption for sector I
 DIEQ(TR,J)    Intermediate consumption of good TR by sector J
 LDEM1(TR)     Labour demand for tradable sectors
 LDEM2(NTR)    Labour demand for non-tradable sectors
 LANDEM        Agricultural land demand
*-------------------------------------------------------------------------------
 LDAGGR(I)     Aggregated labour demande
 LDSUB(I)      Gender Labour Substitution
*-------------------------------------------------------------------------------

*Income and savings
 INCH(H)       Household income (workers)
 INCDH(H)      Household H disposable income
 INCF          Firms income
 INCG          Government income
 SAVH(H)       Household H savings
 SAVF          Firms savings
 SAVG          Government savings

*Taxes
 INDTAX(TR)    Receipts from indirect taxes on TR
 IMDUTY(TR)    Receipts from import duties
 EXTAX(TR)     Receipts from tax on exports
 DIRTAXH(H)    Receipts from household taxation
 DIRTAXF       Receipts from firm taxation

*Demand
CTHEQ(H)      Household H total consumption
 CONSH(TR,H)   Household H consumption of good TR
 CONSG         Public consumption
 INVEST(TR)    Investment in good
 INVVOL        Total investment in volume
 INTDEM(TR)    Intermediate demand

*Prices
 PRVA(I)       Value added price
 RETK1(NAG)    Rate of return to capital from non-agricultural sectors
 RETCF         Return to composite capital-labor factor in agricultural sectors
 RETK2(AGRS)   Rate of return to capital from agricultural sectors
 AVERWAGE      Average wage
 PRDL(TR)      Domestic price
 PRM(TR)       Import prices
 PRE(TR)       Export prices
 PRC(TR)       Composite price (tradables)
 PRP(TR)       Producer price (tradables)
 PII           Price index for investment
 AVPRI         Producer price index

*International trade
 CET(TR)       Relation between D and EX
 EXPORTS(TR)   Export supply
 ARMING(TR)    CES between imports and domestic good
 IMPORT(TR)    Import demand
 CURACC        Current account

*Equilibrium
 DOMABS(GOOD)  Domestic absorption (goods)
 KEQUI         Capital market equilibrium
 REQUI(TR)     Uniform return to capital

*===============================================================================
 SUPFEMLAB(H)  Household H female labour supply
 SUPMALLAB(H)  Household H male labour supply
 TSUPFEMLAB    Total female labour supply
 TSUPMALLAB    Total male labour supply
*===============================================================================

 MALEQUI       Male labour market equilibrium
 FELEQUI       Fem labour market equilibrium
 ISEQUI        Investment-savings equilibrium

*Others
 EVEQ(H)       Calculation of EV
 WALRAS        Verification of the Walras law
 OBJ           Objective function
;


*Production

SUPPLY(I)..       XS(I)    =E=  VA(I)/v(I);

VAD1(NAG)..       VA(NAG)  =E= A_KL(NAG)*(alpha_kl(NAG)*LD(NAG)**(-rho_kl(NAG))

                               +(1-alpha_kl(NAG))*KD(NAG)**(-rho_kl(NAG)))
                               **(-1/rho_kl(NAG));

VAD2..            VA("AGR") =E= A_CL*(alpha_cl*CF**(-rho_cl)+(1-alpha_cl)
                               *LAND**(-rho_cl))**(-1/rho_cl);

ECF..             CF =E= A_KL("AGR")*(alpha_kl("AGR")*LD("AGR")**(-rho_kl("AGR"))
                               +(1-alpha_kl("AGR"))*KD("AGR")**(-rho_kl("AGR")))
                               **(-1/rho_kl("AGR"));

VAD3(NTR)..       VA(NTR)  =E=  LD(NTR);

CIEQ(I)..         CI(I)    =E=  io(I)*XS(I);

DIEQ(TR,J)..      DI(TR,J) =E=  aij(TR,J)*CI(J);

LANDEM..          LAND     =E= (((1-alpha_cl)*rc)/(alpha_cl*rl))**sigma_cl*CF;

LDEM1(TR)..       LD(TR)   =E=  ((alpha_kl(TR)/(1-alpha_kl(TR)))**(sigma_kl(TR))
                                *(r(TR)/w)**sigma_kl(TR))*KD(TR);

LDEM2(NTR)..      LD(NTR)  =E= (P(NTR)*XS(NTR)-SUM(TR,(DI(TR,NTR)*PC(TR))))/w;

LDAGGR(I)..      LD(I)     =E=  A_lab(I)*(alpha_lab(I)*LDfem(I)**(-rho_lab(I))
                 +(1-alpha_lab(I))*LDmal(I)**(-rho_lab(I)))**(-1/rho_lab(I));

LDSUB(I)..       LDfem(I) =E= ((alpha_lab(I)/(1-alpha_lab(I)))*(Wmal/Wfem))**(sigma_lab(I))*LDmal(I);

*Income and savings
*===============================================================================
*INCH(H)..         YH(H)    =E=  lambda_wf(H)*wfem*SUM(I,LDfem(I))+ lambda_wm(H)
*                                *wmal*SUM(I,LDmal(I))+lambda_r(H)
*                                *SUM(TR,r(TR)*KD(TR))+lambda_l(H)*rl*LAND
*                                +PINDEX*TG(H)+DIV(H);

INCH(H)..         YH(H)    =E=  Wfem*LSfem(H)+ Wmal*LSmal(H)+lambda_r(H)
                                *SUM(TR,r(TR)*KD(TR))+lambda_l(H)*rl*LAND
                                +PINDEX*TG(H)+DIV(H);
*===============================================================================

INCDH(H)..        YDH(H)   =E=  YH(H) - DTH(H);

INCF..            YF       =E=  lambda_rf*SUM(TR,r(TR)*KD(TR))
                                +lambda_lf*rl*LAND;

INCG..            YG       =E=  SUM(TR,TI(TR))+SUM(H,DTH(H))+SUM(TR,TIE(TR))+
                                SUM(TR,TIM(TR))+DTF    ;

SAVH(H)..         SH(H)    =E=  nu*psi(H)*YDH(H) ;

SAVF..            SF       =E=  YF - SUM(H,DIV(H)) - DTF - e*DIV_ROW ;

SAVG..            SG       =E=  YG - G - SUM(H,TG(H))*PINDEX;

*Taxes

INDTAX(TR)..      TI(TR)   =E=  (adj+tx(TR)*(1+adj))*(P(TR)*XS(TR)-PE(TR)*EX(TR))+
                                (adj+tx(TR)*(1+adj))*(1+tm(TR))*e*PWM(TR)*M(TR);

IMDUTY(TR)..      TIM(TR)  =E=  tm(TR)*PWM(TR)*e*M(TR);

EXTAX(TR)..       TIE(TR)  =E=  te(TR)*PE(TR)*EX(TR);

DIRTAXH(H)..      DTH(H)   =E=  tyh(H)*YH(H)  ;

DIRTAXF..         DTF      =E=  tyf*YF  ;


*Demand

CTHEQ(H)..        CTH(H) =E= YDH(H)-SH(H);
*===============================================================================

*CONSH(TR,H)..     C(TR,H)*PC(TR) =E= C_MIN(TR,H)*PC(TR)+gamma(TR,H)
*                          *(CTH(H)-SUM(TRJ,C_MIN(TRJ,H)*PC(TRJ)));

CONSH(TR,H)..     C(TR,H)*PC(TR) =E= C_MIN(TR,H)*PC(TR)+(gamma(TR,H)
                                         /(1-ZETA_fem(H)-ZETA_mal(H)))
                                         *(CTH(H)-SUM(TRJ,C_MIN(TRJ,H)*PC(TRJ)));
*===============================================================================

CONSG..           G        =E=  XS("NTSER")* P("NTSER");

INVEST(TR)..      INV(TR)  =E=  mu(TR)*IT/PC(TR) ;

INVVOL..          ITVOL*PINV    =E= IT;

INTDEM(TR)..      DIT(TR)  =E=  SUM(J, DI(TR,J)) ;


*Prices

PRVA(I)..         PV(I)    =E= (P(I)*XS(I)-SUM(TR,DI(TR,I)*PC(TR)))/VA(I);

RETK1(NAG)..      R(NAG)   =E= (PV(NAG)*VA(NAG) - w*LD(NAG) )/KD(NAG) ;

RETK2(AGRS)..     R("AGR")*KD("AGR") =E= RC*CF-w*LD("AGR");

RETCF..           RC       =E= (PV("AGR")*VA("AGR") - rl*LAND)/CF;

AVERWAGE..         w     =E= (wfem*SUM(I,LDfem(I))+wmal*SUM(I,LDmal(I)))/SUM(I,LD(I));

PRDL(TR)..        PD(TR)   =E=  PL(TR)*(1+tx(TR))*(1+adj);

PRM(TR)..         PM(TR)   =E=  (1+TX(TR))*(1+adj)*(1+tm(TR))*e*PWM(TR);

PRE(TR)..         PE(TR)   =E=  PWE(TR)*e/(1+te(TR));

PRC(TR)..         PC(TR)   =E=  (PD(TR)*D(TR)+PM(TR)*M(TR))/Q(TR);

PRP(TR)..         P(TR)    =E=  (PL(TR)*D(TR) + PE(TR)*EX(TR))/XS(TR);

PII..             PINV     =E=  PROD(TR$(INVO(TR) NE 0), (PC(TR)/mu(TR))**mu(TR));

AVPRI..           PINDEX   =E=  SUM(I,PV(I)*delta(I));


*International trade

CET(TR)..         XS(TR)   =E=  B_E(TR)*(beta_e(TR)*EX(TR)**kappa_e(TR)
                               +(1-beta_e(TR))*D(TR)**kappa_e(TR))
                               **(1/kappa_e(TR));

EXPORTS(TR)..     EX(TR)   =E=  ((PE(TR)/PL(TR))**tau_e(TR)
                                *((1-beta_e(TR))/beta_e(TR))**tau_e(TR))*D(TR);

ARMING(TR)..      Q(TR)    =E=  A_M(TR)*(alpha_m(TR)*M(TR)**(-rho_m(TR))
                                +(1-alpha_m(TR))*D(TR)**(-rho_m(TR)))
                                **(-1/rho_m(TR));

IMPORT(TR)..      M(TR)    =E=  ((alpha_m(TR)/(1-alpha_m(TR)))**(sigma_m(TR))
                                *(PD(TR)/PM(TR))**sigma_m(TR))*D(TR);

CURACC..          CAB      =E=  SUM(TR,PWM(TR)*M(TR))+ lambda_row*SUM(TR,r(TR)*KD(TR))/e
                                +lambda_lrw*rl*LAND/e+DIV_ROW-SUM(TR,PWE(TR)*EX(TR));

*Equilibrium

DOMABS(GOOD)..    Q(GOOD) =E=   SUM(H,C(GOOD,H))+DIT(GOOD)+INV(GOOD);

KEQUI..           KS      =E=   SUM(TR,KD(TR));

REQUI(TR)..       r(TR)   =E=   rf;

*===============================================================================
SUPFEMLAB(H)..    Wfem*LSfem(H) =E= Wfem*MAXHOUR_Fem(H)-ZETA_fem(H)
                                 *(CTH(H)-SUM(TRJ,C_MIN(TRJ,H)*PC(TRJ)))
                                 /(1-ZETA_fem(H)-ZETA_mal(H));
SUPMALLAB(H)..    Wmal*LSmal(H) =E= Wmal*MAXHOUR_mal(H)-ZETA_mal(H)
                                 *(CTH(H)-SUM(TRJ,C_MIN(TRJ,H)*PC(TRJ)))
                                 /(1-ZETA_fem(H)-ZETA_mal(H));

TSUPFEMLAB..      LSTfem  =E= SUM(H,LSfem(H));

TSUPMALLAB..      LSTmal  =E= SUM(H,LSmal(H));
*===============================================================================

MALEQUI..         LSTmal      =E=   SUM(I,LDmal(I));

FELEQUI..         LSTfem      =E=   SUM(I,LDfem(I));

ISEQUI..          IT      =E=   SUM(H,SH(H)) + SF + SG + CAB*e;


*Others

EVEQ(H)..         EV(H)   =E=  PROD(TR,(PCO(TR)/PC(TR))**gamma(TR,H))
                          *(CTH(H)-SUM(TRJ,C_MIN(TRJ,H)*PC(TRJ)))
                          -(CTHO(H)-SUM(TRJ,C_MIN(TRJ,H)*PCO(TRJ)));

WALRAS..          LEON    =E=   Q("SER")- SUM(H,C("SER",H))-DIT("SER");

OBJ..             OMEGA   =E=   1000;

*----------------------------------Initialization----------------------------*

w.L         =  wo;
wfem.L      =  wfemo;
wmal.L      =  wmalo;
r.L(TR)     =  ro(TR);
rl.l        =  rlo;
rf.l        =  1.0;
rc.l        =  rco;
P.L(I)      =  PO(I);
PD.L(TR)    =  PDO(TR);
PV.L(I)     =  PVO(I);
PL.L(TR)    =  PLO(TR);
PC.L(TR)    =  PCO(TR);
PM.L(TR)    =  PMO(TR);
PE.L(TR)    =  PEO(TR);
PWM.L(TR)   =  PWMO(TR);
PWE.L(TR)   =  PWEO(TR);
PINDEX.L    =  PINDEXO;
PINV.L      =  PINVO;
e.L         =  eO;

XS.L(I)     =  XSO(I);
VA.L(I)     =  VAO(I);
DI.L(TR,J)  =  DIO(TR,J);
CI.L(I)     =  CIO(I);

KD.L(TR)    =  KDO(TR);
KS.L        =  KSO;
LD.L(I)     =  LDO(I);
LAND.L      =  LANDO;
CF.L        =  CFO;

LDfem.L(I)  =  LDfemO(I);
LDmal.L(I)  =  LDmalO(I);

C.L(TR,H)   =  CO(TR,H);
CTH.L(H)    =  CTHO(H);
INV.L(TR)   =  INVO(TR);
IT.L        =  ITO;
ITVOL.L     =  ITVOLO;
DIT.L(TR)   =  DITO(TR);
G.L         =  GO;
D.L(TR)     =  DO(TR);
Q.L(TR)     =  QO(TR);

M.L(TR)     =  MO(TR);
EX.L(TR)    =  EXO(TR);
CAB.L       =  CABO;

YH.L(H)     =  YHO(H);
YDH.L(H)    =  YDHO(H);
YF.L        =  YFO;
YG.L        =  YGO ;
SH.L(H)     =  SHO(H);
SF.L        =  SFO;
SG.L        =  SGO;
DIV.L(H)    =  DIVO(H);
DIV_ROW.L   =  DIV_ROWO;
TG.L(H)     =  TGO(H);
TI.L(TR)    =  TIO(TR);
TIM.L(TR)   =  TIMO(TR);
TIE.L(TR)   =  TIEO(TR);
DTH.L(H)    =  DTHO(H);
DTF.L       =  DTFO;
nu.L        =  1;
adj.L       =  0;

LEON.L      =  0;
OMEGA.L     =  1000;

LD.LO(I)    =  LDO(I)*0.001;
D.LO(TR)    =  DO(TR)*0.001;
M.LO(TR)    =  MO(TR)*0.001;
EX.LO(TR)   =  EXO(TR)*0.001;
*-------------------------------------Closure--------------------------------*

* Exchange rate is the numeraire, capital is sector specific, fixed public
* expenditure (in volume), fixed investment (in volume) CAB fixed.



*LSfem.FX    = LSfemO;
*LSmal.FX    = LSmalO;
KS.FX       = KSO;
*KD.FX("AGR")= KDO("AGR");
*KD.FX("SER")= KDO("SER");
*KD.FX("IND")= KDO("IND");
LAND.FX      = LANDO;
ITVOL.FX     = ITO/PINVO;
DIV.FX(H)    = DIVO(H);
DIV_ROW.FX  = DIV_ROWO;
TG.FX(H)    = TGO(H);
XS.FX(NTR) = XSO(NTR);
*G.FX        = GO;
PWM.FX(TR)  = PWMO(TR);
PWE.FX(TR)  = PWEO(TR);
e.FX        = eo;
CAB.FX      = CABO;
*SG.FX      = SGO;
*YG.FX      = YGO;
*tm(TR)     = 0;
*adj.L      = 0;
adj.fx      = 0;
*......................Endogenous Labour Supply .................................
LSfem.L(H)    = LSfemO(H);
LSmal.L(H)    = LSmalO(H);
LSTfem.L    = LSTfemO;
LSTmal.L    = LSTmalO;
*================== Simulation =================================================

*First scenario : Decreasing in tariff rate for industrial import goods

*tm("IND")  = 0.0*tm("IND");

*Second scenario : Decrease in export taxe rate

*te(TR)  = 0.0*te(TR) ;

*----------------------------MODEL EXECUTION-------------------------------*

OPTION NLP=minos;
option limrow=1;
MODEL EXTERplusGLS ENDOGENOUS MALE AND FEMALE LABOUR SUPPLY   /ALL/;
EXTERplusGLS.HOLDFIXED=1;
SOLVE EXTERplusGLS MAXIMIZING OMEGA USING NLP;
$INCLUDE gls_sim_tables.gms

*----------------------------------------------------------------------------*

